/* BBAltMenu - a plugin of BB4W to provide alternative menu 
   Copyright (C) 2004 kana <nicht AT s8 DOT xrea DOT con>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <BBApi.h>

#define NUMBER_OF(a) (sizeof(a) / sizeof((a)[0]))




static const char* const BBAM_NAME = "BBAltMenu";
static const char* const BBAM_VERSION = "0.0.0b";
static const char* const BBAM_AUTHOR = "kana";
static const char* const BBAM_RELEASE = "2004-07-15/24";
static const char* const BBAM_LINK
                           = "http://nicht.s8.xrea.com/2004/07/15/BBAltMenu";
static const char* const BBAM_EMAIL = "nicht AT s8 DOT xrea DOT con";

static const UINT BBAM_MESSAGES[] = {BB_BROADCAST, 0};




static struct {
	HWND window;
	HINSTANCE instance;
} G;








static void
error(const char* format, ...)
{
	char buf[80*25];
	va_list va;

	va_start(va, format);
	vsnprintf(buf, NUMBER_OF(buf), format, va);
	va_end(va);

	MessageBox( G.window, buf, BBAM_NAME,
	            MB_OK | MB_ICONERROR
	              | MB_SETFOREGROUND | MB_TASKMODAL | MB_TOPMOST );
}


static void
SNPRINTF(char* buf, int size, const char* format, ...)
{
	va_list va;

	va_start(va, format);
	vsnprintf(buf, size, format, va);
	va_end(va);

	buf[size - 1] = '\0';
}




static FILE*
open_file(const char* filename)
{
	const HINSTANCE INSTANCE_TABLE[] = {G.instance, NULL};
	char dir[MAX_PATH];
	char path[MAX_PATH];
	char* p;
	int i;
	FILE* f;

	if (strchr(filename, '\\') != NULL) {
		return fopen(filename, "rt");
	} else {
		for (i = 0; i < (int)NUMBER_OF(INSTANCE_TABLE); i++) {
			if (!GetModuleFileName( G.instance,
			                        dir, NUMBER_OF(dir) ))
			{
				continue;
			}

			p = strrchr(dir, '\\');
			if (p != NULL) p[1] = '\0';

			SNPRINTF(path, NUMBER_OF(path), "%s%s", dir, filename);

			f = fopen(path, "rt");
			if (f != NULL)
				return f;
		}

		return NULL;
	}
}




static Menu*
parse_menu(FILE* f, Menu* parent, const char* file_name, int* no)
{
	char buf[1024];
	char command[NUMBER_OF(buf)];
	char label[NUMBER_OF(buf)];
	char data[NUMBER_OF(buf)];
	char menu_id[80];
	char* line;
	int n;

	while (fgets(buf, NUMBER_OF(buf), f) != NULL) {
		line = buf;

		/* skip comment */
		while (*line == ' ' || *line == '\t') line++;
		if (*line == '#' || *line == '!') continue;

		n = sscanf( line, "[%[^][]] (%[^()]) {%[^{}]}",
		            command, label, data );
		if (!(1 <= n && n <= 3)) continue;

		if (parent == NULL && !strcmp(command, "begin")) {
			(*no)++;
			SNPRINTF( menu_id, NUMBER_OF(menu_id), "%s/%s/%d",
				  BBAM_NAME, file_name, *no );

			parent = MakeNamedMenu( (2 <= n ? label : BBAM_NAME),
			                        menu_id, true );
			parse_menu(f, parent, file_name, no);
		} else if (!strcmp(command, "end")) {
			return parent;
		} else if (parent == NULL) {
			; /* don't process the following if parent == NULL */
		} else if (!strcmp(command, "submenu") && n == 2) {
			Menu* child;

			(*no)++;
			SNPRINTF( menu_id, NUMBER_OF(menu_id), "%s/%s/%d",
				  BBAM_NAME, file_name, *no );

			child = MakeNamedMenu(label, menu_id, true);
			parse_menu(f, child, file_name, no);
			MakeSubmenu(parent, child, label);
		} else if (!strcmp(command, "nop") && n <= 2) {
			MakeMenuNOP(parent, (n == 2 ? label : ""));
		} else if (!strcmp(command, "exec") && n == 3) {
			MakeMenuItem(parent, label, data, false);
		} else {
			; /* ignore */
		}
	}

	return parent;
}


static void
show_menu(const char* alt_menu_rc)
{
	FILE* f;
	Menu* menu;
	int no;

	f = open_file(alt_menu_rc);
	if (f == NULL) {error("`%s' not found", alt_menu_rc);  return;}

	no = 0;
	menu = parse_menu(f, NULL, alt_menu_rc, &no);

	fclose(f);

	if (menu != NULL)
		ShowMenu(menu);
}








static LRESULT CALLBACK
window_procedure(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg) {
	default:
		break;


	case BB_BROADCAST: {
		const char* s;

		s = (const char*)lp;

		if (!!strncmp(s, "@BBAltMenu.", 11)) break;
		s += 11;

		if (!strncmp(s, "open ", 5)) {
			s += 5;

			show_menu(s);
		}

		} break;


	case WM_CREATE:
		SendMessage( GetBBWnd(), BB_REGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBAM_MESSAGES );
		break;

	case WM_DESTROY:
		SendMessage( GetBBWnd(), BB_UNREGISTERMESSAGE,
		             (WPARAM)hwnd, (LPARAM)BBAM_MESSAGES );
		break;
	}

	return DefWindowProc(hwnd, msg, wp, lp);
}








extern "C" DLL_EXPORT int
beginPlugin(HINSTANCE plugin_instance)
{
	WNDCLASS wc;

	G.instance = plugin_instance;

	memset(&wc, 0x00, sizeof(wc));
	wc.lpfnWndProc = window_procedure;
	wc.hInstance = plugin_instance;
	wc.lpszClassName = BBAM_NAME;
	if (!RegisterClass(&wc))
		{error("RegisterClass failed");  goto E_REGISTER_CLASS;}

	G.window = CreateWindowEx( WS_EX_TOOLWINDOW,
	                           BBAM_NAME,
	                           BBAM_NAME,
	                           0,
	                           0, 0, 0, 0,
	                           HWND_MESSAGE,
	                           NULL,
	                           plugin_instance,
	                           NULL );
	if (G.window == NULL)
		{error("CreateWindowEx failed");  goto E_CREATE_WINDOW_EX;}

	return 0;


E_CREATE_WINDOW_EX:
	UnregisterClass(BBAM_NAME, plugin_instance);
E_REGISTER_CLASS:
	return !0;
}




extern "C" DLL_EXPORT void
endPlugin(HINSTANCE plugin_instance)
{
	DestroyWindow(G.window);  G.window = NULL;
	UnregisterClass(BBAM_NAME, plugin_instance);
}




extern "C" DLL_EXPORT LPCSTR
pluginInfo(int field)
{
	switch (field) {
	case PLUGIN_NAME:    return BBAM_NAME;
	case PLUGIN_VERSION: return BBAM_VERSION;
	case PLUGIN_AUTHOR:  return BBAM_AUTHOR;
	case PLUGIN_RELEASE: return BBAM_RELEASE;
	case PLUGIN_LINK:    return BBAM_LINK;
	case PLUGIN_EMAIL:   return BBAM_EMAIL;
	default:             return BBAM_NAME;
	}
}




/* __END__ */
